@@ -3,7 +3,7 @@ package Assert::Refute::Report;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 =head1 NAME
 
@@ -54,26 +54,17 @@ my $ERROR_DONE = "done_testing was called, no more changes may be added";
 
 =head2 new
 
-    Assert::Refute::Report->new( %options );
+    Assert::Refute::Report->new();
 
-%options may include:
-
-=over
-
-=item * indent - log indentation (will be shown as 4 spaces in C<get_tap>);
-
-=back
+No arguments are currently supported.
 
 =cut
 
 sub new {
-    my ($class, %opt) = @_;
-
     bless {
-        indent => $opt{indent} || 0,
         fail   => {},
         count  => 0,
-    }, $class;
+    }, shift;
 };
 
 =head2 RUNNING PRIMITIVES
@@ -210,7 +201,7 @@ sub done_testing {
     };
 
     # Any post-mortem messages go to a separate bucket
-    $self->{log} = $self->{messages}{ $self->{count}+1 } = [];
+    $self->{log} = $self->{messages}{ -1 } = [];
 
     if (defined $self->{plan_tests}) {
         # Check plan
@@ -390,13 +381,18 @@ Returns a hash containing information about a test:
 
 =over
 
+=item number - the number of test (this is equal to argument);
+
 =item ok - whether the test was successful;
 
-=item name - name of the test.
+=item name - name of the test (if any);
+
+=item reason - the reason for test failing, if it failed;
 
-=item reason - the reason for test failing, if it failed.
 Undefined for "ok" tests.
 
+=item diag - diagnostic messages as one array, without leading C<#>;
+
 =item log - any log messages that followed the test (see get_log for format)
 
 =item subcontract - if test was a subcontract, contains the report
@@ -405,8 +401,9 @@ Undefined for "ok" tests.
 
 Returns empty hash for nonexistent tests, and dies if test number is not integer.
 
-As a special case, test number 0 only contains the log
-for C<diag>/C<note> messages that were written before any tests.
+As a special case, tests number 0 and -1 represent the output before any
+tests and postmortem output, respectively.
+These only contains the C<log> and C<diag> fields.
 
 See also L<Test::Tester>.
 
@@ -418,7 +415,7 @@ sub get_result_details {
     my ($self, $n) = @_;
 
     $self->_croak( "Bad test number $n, must be nonnegatine integer" )
-        unless defined $n and $n =~ /^[0-9]+$/;
+        unless defined $n and $n =~ /^(?:[0-9]+|-1)$/;
 
     # Process messages, return if premature(0) or post-mortem (n+1)
     my @messages;
@@ -426,27 +423,33 @@ sub get_result_details {
         @messages = @$array;
     };
 
-    if ($n < 1 or $n > $self->{count} ) {
-        return { log => \@messages };
+    my %ret = ( number => $n );
+
+    if ($n >= 1) {
+        # a real test - add some information
+        my $reason = $self->{fail}{$n};
+        my @diag;
+
+        if (ref $reason eq 'ARRAY') {
+            push @diag, [ 0, -1, to_scalar($_) ] for @$reason;
+        } elsif ( $reason and $reason ne 1 ) {
+            push @diag, [ 0, -1, to_scalar($reason) ];
+        };
+
+        $ret{ok}          = !$reason;
+        $ret{name}        = $self->{name}{$n};
+        $ret{reason}      = $reason;
+        $ret{log}         = [@diag, @messages];
+        $ret{subcontract} = $self->{subcontract}{$n};
+    } else {
+        # leading or trailing messages
+        $ret{log} = \@messages,
     };
 
-    # now analyze the test itself
-    my $reason = $self->{fail}{$n};
-    my @diag;
+    # Strip extra trash from internal log format
+    $ret{diag} = [ map { $_->[2] } grep { $_->[1] < 0 } @{ $ret{log} } ];
 
-    if (ref $reason eq 'ARRAY') {
-        push @diag, [ $self->{indent}, -1, to_scalar($_) ] for @$reason;
-    } elsif ( $reason and $reason ne 1 ) {
-        push @diag, [ $self->{indent}, -1, to_scalar($reason) ];
-    };
-
-    return {
-        ok          => !$reason,
-        name        => $self->{name}{$n},
-        reason      => $reason,
-        log         => [@diag, @messages],
-        subcontract => $self->{subcontract}{$n},
-    };
+    return \%ret;
 };
 
 =head3 get_error
@@ -632,8 +635,6 @@ sub do_log {
     $self->_croak( $ERROR_DONE )
         if $self->{done};
 
-    $indent += $self->{indent};
-
     $self->{log} ||= $self->{messages}{ $self->{count} } ||= [];
     push @{ $self->{log} }, [$indent, $level, $mess];
 
@@ -659,19 +660,19 @@ sub get_log {
 
     # output plan if there was plan
     if (defined $self->{plan_tests}) {
-        push @mess, [ $self->{indent}, 0, "1..$self->{plan_tests}" ]
+        push @mess, [ 0, 0, "1..$self->{plan_tests}" ]
             unless $verbosity < 0;
     };
 
-    foreach my $n ( 0 .. $self->{count} + 1 ) {
+    foreach my $n ( 0 .. $self->{count}, -1 ) {
         # Report test details.
         # Only append the logs for
         #   premature (0) and postmortem (count+1) messages
-        if ($n and $n <= $self->{count}) {
+        if ($n > 0) {
             my $reason = $self->{fail}{$n};
             my ($level, $prefix)  = $reason ? (-2, "not ok") : (0, "ok");
             my $name   = $self->{name}{$n} ? "$n - $self->{name}{$n}" : $n;
-            push @mess, [ $self->{indent}, $level, "$prefix $name" ];
+            push @mess, [ 0, $level, "$prefix $name" ];
 
             if ($self->{subcontract}{$n}) {
                 push @mess, map {
@@ -681,10 +682,10 @@ sub get_log {
 
             if (ref $reason eq 'ARRAY') {
                 push @mess, map {
-                    [ $self->{indent}, -1, to_scalar( $_ ) ]
+                    [ 0, -1, to_scalar( $_ ) ]
                 } @$reason;
             } elsif ($reason and $reason ne 1) {
-                push @mess, [ $self->{indent}, -1, to_scalar( $reason ) ];
+                push @mess, [ 0, -1, to_scalar( $reason ) ];
             };
         };
 
@@ -695,7 +696,7 @@ sub get_log {
     };
 
     if (!defined $self->{plan_tests} and $self->{done}) {
-        push @mess, [ $self->{indent}, 0, "1..".$self->get_count ]
+        push @mess, [ 0, 0, "1..".$self->get_count ]
             unless $verbosity < 0;
     };
 
